<?php

/**
 * @file all functions needed to make erpal_crm accessible by services
 * @author Manuel Pistner <pistner@brightsolutions.de>
 * @copyright (c) 2013
 */

/**
 * Create a new crm_activity node by services module request
 * @param array $activity the data delivered by the webservice, $activity['crm_activity'] must be an array
 * @return the activity that was created or an error of activity could not be created
 * @TODO: maybe use drupal_form_submit or _node_resource_create to have validation instead of node_save
 */
function _erpal_crm_service_crm_activity_create($activity) {
  $returns = array();
  global $user;

  $value = $activity['crm_activity'];

  //if we don't have title, take the first 65 chars from body
  $body = isset($value['body']) ? $value['body'] : '';
  $title = isset($value['title']) ? $value['title'] : substr($body, 1, 65);
  $origin = isset($value['origin']) ? $value['origin'] : '';
  $customer_nid = isset($value['customer_nid']) ? $value['customer_nid'] : false;

  //Find the term to the origin string
  //@TODO: maybe better escape the origin?
  $origin_term_id = taxonomy_get_term_by_name($origin, 'activity_origin');
  $origin_term_id = reset($origin_term_id);
  $origin_term_id = $origin_term_id->tid;

  //@TODO: set next contact and make it configurable in ERPAL settings

  $form_id = 'erpal_crm_activity_node_form';
  $form_values = array(
    'title' => $title,
    'body' => array(LANGUAGE_NONE => array(0 => array('value' => $body, 'format' => 'filtered_html'))),
    'language' => LANGUAGE_NONE,
    'name' => $user->name,
    'status' => 1,
    'type' => 'erpal_crm_activity',
    'promote' => 0,
    'sticky' => 0,
  );

  if ($origin_term_id) {
    $form_values['field_activity_origin'][LANGUAGE_NONE]['tid'] = $origin_term_id;
  }

  if ($customer_nid) {
    $form_values['field_customer_ref'][LANGUAGE_NONE][0]['target_id'] = _erpal_basic_helper_autocomplete_string_from_nid($customer_nid);
  }

  // node module stores form processing functions in a
  // seperate include entitled node.pages.inc
  // so must load that before executing form
  module_load_include('inc', 'node', 'node.pages');
  $form_values['op'] = t('Save');  // this seems to be a required value
  $form_state['values'] = $form_values;

  $activity_node = array('type' => 'erpal_crm_activity');
  drupal_form_submit($form_id, $form_state, (object) $activity_node);
  $node = $form_state['node'];

  $errors = form_get_errors();
  //die(print_r($form_state['node']));
  return _erpal_crm_service_create_service_response($errors, $activity_node);
}

/**
 * callback for retrieving a single activity resource.
 *
 * @param int $id the nid of the project node that should be retrieved
 * @param bool $inlude_children, instead of url, include short version of children, defaults to false
 * @return object
 */
function _erpal_crm_service_crm_activity_retrieve($id, $details = TRUE) {
  $contact_node = _erpal_crm_service_crm_activity_out((int) $id, $details);
  return $contact_node;
}

/**
 * Callback for returning a list of activities respecting given filters
 * @param int $start number to start list from, defaults to 0
 * @param int $count number of item to return, defaults to PHP_INT_MAX
 * @param int $changed_since UTC timestamp, deliver only tasks changed after that timestamp , defaults to 0
 * @param bool $details whether tasks will be returned with details , defaults to false
 * @return array array of tasks, prepared for output
 */
function _erpal_crm_service_crm_activity_index($start = 0, $count = PHP_INT_MAX, $changed_since = 0, $details = false) {

  $query = new EntityFieldQuery();
  $query->entityCondition('entity_type', 'node')
      ->propertyCondition('type', 'erpal_crm_activity')
      ->range((int) $start, (int) $count)
      ->propertyCondition('changed', (int) $changed_since, '>=');

  $entities = $query->execute();

  $activities_out = array();

  if (!empty($entities['node'])) {
    foreach (array_keys($entities['node']) as $nid) {
      $activities_out[] = _erpal_crm_service_crm_activity_out($nid, $details);
    }
  }

  return $activities_out;
}

/**
 * retrieves an activity node and returns only simple data
 * @param int $nid the node id of the project to return
 * @param bool $details whether or not to return the project with details
 * @param bool $include_children wheter or not no include children
 * @return array the project prepared for output
 */
function _erpal_crm_service_crm_activity_out($nid, $details = FALSE) {

  // load activity node
  $activity = node_load($nid);
  //dd($activity, 'activity_out');

  // check if node is an activity
  if ($activity->type !== 'erpal_crm_activity') {
    return services_error('failed loading node. nid ' . $nid . ' is not an activity', 400);
  }

  // load wrapper
  $activity_wrapper = entity_metadata_wrapper('node', $activity);

  // base data
  $activity_out = array();
  $activity_out['created'] = (int) $activity->created;
  $activity_out['changed'] = (int) $activity->changed;
  $activity_out['url'] = url('rest/activity/crm_activity/' . $activity->nid, array('absolute' => TRUE));
  $activity_out['title'] = $activity->title;

  // revision
  $activity_out['revision_timestamp'] = (int) $activity->revision_timestamp;
  $activity_out['revision_id'] = (int) $activity->revision_uid;

  if ($details) {

    // body
    if (!empty($activity->body)) {
      $activity_out['body'] = strip_tags($activity_wrapper->body->value->value());
    }

    // status
    if (!empty($activity->field_activity_status_term)) {
      $activity_out['status'] = $activity_wrapper->field_activity_status_term->name->value();
    }

    // next contact
    if (!empty($activity->field_next_contact)) {
      $activity_out['next_contact'] = (int) $activity_wrapper->field_next_contact->value();
    }

    // customer
    if (!empty($activity->field_customer_ref)) {
      $customer_id = $activity->field_customer_ref[LANGUAGE_NONE][0]['target_id'];
      $activity_out['customer'] = url('rest/basic/contact/' . $customer_id, array('absolute' => TRUE));
    }

    // contacts
    $activity_out['contacts'] = array();
    if (!empty($activity->field_contacts_ref)) {
      foreach ($activity->field_contacts_ref[LANGUAGE_NONE] as $key => $value) {
        $contact_id = $value['target_id'];
        $contact_url = url('rest/basic/contact/' . $contact_id, array('absolute' => TRUE));
        array_push($activity_out['contacts'], $contact_url);
      }
    }

    // files
    $activity_out['files'] = array();
    foreach ($activity_wrapper->field_asset_files->getIterator() as $key => $file) {
      $file_array = $file->field_docs_file->value();
      if (!empty($file_array)) {
        array_push($activity_out['files'], array(
          'filename' => $file_array['filename'],
          'mimetype' => $file_array['filemime'],
          'filesize' => (int) $file_array['filesize'],
          'url' => file_create_url($file_array['uri']),
          'description' => ($file_array['description']) ? $file_array['description'] : '',
          'fid' => (int) $file_array['fid'],
        ));
      }
    }

    // comments
    $activity_out['comments'] = array();
    $query = new EntityFieldQuery();
    $query->entityCondition('entity_type', 'comment')
        ->propertyCondition('nid', $nid);
    $entities = $query->execute();
    if (!empty($entities['comment'])) {
      foreach (array_keys($entities['comment']) as $cid) {
        $comment = comment_load($cid);
        if ($comment) {
          $comment_body = $comment->comment_body[LANGUAGE_NONE][0];
          array_push($activity_out['comments'], array(
            'created' => (int) $comment->created,
            'changed' => (int) $comment->changed,
            'cid' => (int) $comment->cid,
            'subject' => ($comment->subject) ? $comment->subject : '',
            'name' => ($comment->name) ? $comment->name : '',
            'registered_name' => ($comment->registered_name) ? $comment->registered_name : '',
            'comment_body' => $comment_body['value'],
            'comment_format' => $comment_body['format'],
          ));
        }
      }
    }
  }

  return $activity_out;
}

/**
 * Check access for creating a crm_activity node
 */
function _erpal_crm_service_crm_activity_access($op, $args) {
  return node_access($op, 'erpal_crm_activity');
}

/**
 * Creates an array as the response of a webservice request
 */
function _erpal_crm_service_create_service_response($errors, $activity_node) {
  if (!empty($errors)) {
    $returns['error'] = TRUE;
    $error_string = '';
    foreach ($errors as $key => $value) {
      $error_string .= $key . ": " . $value . "<br/>";
    }
    $returns['message'] = 'Activity could not be created because of some errors: <br/>' . $error_string;
  }
  else {
    $returns['node'] = (array) $activity_node;
    $returns['error'] = false;
    $returns['message'] = '';
  }

  return $returns;
}